#include <Wire.h>
#include <SparkFun_APDS9960.h>
 
#define APDS9960_INT 2
#define RED_LED PD6 // 6
#define RED_PIN 6
#define YELLOW_LED PB1 // 11
#define YELLOW_PIN 11
#define BLUE_LED PD5 // 5
#define BLUE_PIN 5
#define POTENTIOMETER_PIN A3
#define BUTTON PD4
#define TIME_LIMIT 40
 
// Global Variables
SparkFun_APDS9960 apds = SparkFun_APDS9960();
int isr_flag = 0;
int lights = 0;
int seconds = 0;
int counter = 0;
int buttonState = HIGH;
int lastButtonState = HIGH;
unsigned long lastDebounceTime = 0;
unsigned long debounceDelay = 100;
 
void setup() {
 
  // Initialize Serial port
  Serial.begin(9600);
 
  // Initialize interrupts
  setup_interrupts();
  
  // Set as input and enable pull-up resistance
  DDRD &= ~(1 << BUTTON);
  PORTD |= (1 << BUTTON);
  ADMUX = 0;
  /* ADC3 - channel 3 */
  ADMUX |= (1 << MUX1) | (1 << MUX0);
  /* AVCC with external capacitor at AREF pin */
  ADMUX |= (1 << REFS0);

  ADCSRA = 0;
  /* set prescaler at 128 */
  ADCSRA |= (7 << ADPS0);
  /* enable ADC */
  ADCSRA |= (1 << ADEN);
  // Set as outputs
  DDRD |= (1 << RED_LED);
  DDRB |= (1 << YELLOW_LED);
  DDRD |= (1 << BLUE_LED);
 
  // Initialise gesture sensor
  if ( apds.init() ) {
    Serial.println(F("APDS-9960 initialization complete"));
  } else {
    Serial.println(F("Something went wrong during APDS-9960 init!"));
  }
 
  if ( apds.enableGestureSensor(true) ) {
    Serial.println(F("Gesture sensor is now running"));
  } else {
    Serial.println(F("Something went wrong during gesture sensor init!"));
  }
}

void setup_interrupts() {
  cli();
  
  // Extern interrupt
  EIMSK |= (1 << INT0);
  // Falling edge of INT0 generates an interrupt request
  EICRA &= ~(1 << ISC00);
  EICRA |= (1 << ISC01);

  // Enable pin change interrupt
  PCMSK2 |= (1 << PCINT20);
  // Set PCIE2 to enable PCMSK2 scan
  PCICR |= (1 << PCIE2);

  TCCR1A = 0;
  TCCR1B = 0;
  TCNT1 = 0;
  OCR1A = 62499; // compare match register
  TCCR1B |= (1 << WGM12);   // CTC mode
  TCCR1B |= (1 << CS12); // 256 prescaler 
  TIMSK1 |= (1 << OCIE1A); // enable interrupts TIMER1_COMPA
  sei();
}


void loop() {
  /* Start conversion */
  ADCSRA |= (1 << ADSC);
  /* Wait until conversion is complete */
  while(!(ADCSRA & ( 1 << ADIF)));
  /* Save result */
  uint16_t result = ADC;
  /* Get brightness */
  int brightness = ADC / 4;
  /* If brightness too small, set it at 50 */
  if (brightness <= 50) {
    brightness = 50;
  }

  /* If counter odd number, lamp is on
  if the seconds limit has not been reached
  execute tasks */
  if (counter % 2 == 1 && seconds < TIME_LIMIT) {
    // Handle gesture sensor
    if( isr_flag == 1 ) {
      lights = handleGesture();
      isr_flag = 0;
    }

    switch (lights) {
        case 1:
        // Turn on all LEDs
          analogWrite(RED_PIN, brightness);
          analogWrite(YELLOW_PIN, brightness);
          analogWrite (BLUE_PIN, brightness);
          break;
        case 2:
        // Turn on yellow LED
          analogWrite(RED_PIN, 0);
          analogWrite (BLUE_PIN, 0);
          analogWrite(YELLOW_PIN, brightness);
          break;
        case 3:
        // Turn on blue LED
          analogWrite(RED_PIN, 0);
          analogWrite(YELLOW_PIN, 0);
          analogWrite (BLUE_PIN, brightness);
          break;
        case 4:
        // Turn on red LED
          analogWrite(YELLOW_PIN, 0);
          analogWrite (BLUE_PIN, 0);
          analogWrite(RED_PIN, brightness);
          break;
    }
  } else {
    // Turn off all LEDs
    analogWrite(YELLOW_PIN, 0);
    analogWrite (BLUE_PIN, 0);
    analogWrite(RED_PIN, 0);
  }
}

ISR(TIMER1_COMPA_vect) {
  // Interrupt code for TIMER1
  get_time();
}

void get_time() {
  // Increments number of seconds
  seconds++;
  // If the limit is reached
  if (seconds >= TIME_LIMIT) {
    // Turn off all LEDs
    analogWrite(RED_PIN, 0);
    analogWrite (BLUE_PIN, 0);
    analogWrite(YELLOW_PIN, 0);
    // Give counter even value for knowing
    // that the lamp is off
    counter = 0;
  }
}

ISR(INT0_vect)
{
  // Interrupt for gesture sensor
  isr_flag = 1;
}

ISR(PCINT2_vect) {
  // Debounce for button
  // Check if enough seconds has passed since the last
  // button press to debounce the input
  if (millis() - lastDebounceTime > debounceDelay) {
    // If the button was previously pressed
    // and the button is currently pressed
    // it means a valid button press has occurred
    if (buttonState == HIGH && digitalRead(4) == LOW) {
      counter++;
      seconds = 0;
    }
    // Update with the current state of the button
    buttonState = digitalRead(4);
    // Update seconds of the latest button press 
    lastDebounceTime = millis();
  }
}
 
int handleGesture() {
    if ( apds.isGestureAvailable() ) {
      // Return by case of gesture
    switch ( apds.readGesture() ) {
      case DIR_UP:
        seconds = 0;
        return 1;
      case DIR_DOWN:
        seconds = 0;
        return 2;
      case DIR_LEFT:
        seconds = 0;
        return 3;
      case DIR_RIGHT:
        seconds = 0;
        return 4;
      default:
        Serial.println("NONE");
    }
  }
}